
{*******************************************************}
{                                                       }
{       RichView                                        }
{       How to print images using OnPagePrepaint event  }
{                                                       }
{       Copyright (c) Sergey Tkachenko                  }
{       svt@trichview.com                               }
{       http://www.trichview.com                        }
{                                                       }
{*******************************************************}


unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, PtblRV, RVStyle, CRVPP, RVPP, RVScroll, RichView, RVFuncs,
  PtRVData, ExtCtrls;

type
  TForm1 = class(TForm)
    RichView1: TRichView;
    RVPrintPreview1: TRVPrintPreview;
    RVStyle1: TRVStyle;
    RVPrint1: TRVPrint;
    ScrollBar1: TScrollBar;
    Image1: TImage;
    Image2: TImage;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure ScrollBar1Change(Sender: TObject);
    procedure RVPrint1PagePrepaint(Sender: TRVPrint; PageNo: Integer;
      Canvas: TCanvas; Preview: Boolean; PageRect, PrintAreaRect: TRect);
    procedure Button1Click(Sender: TObject);
  private
    { Private declarations }
    PrinterHPixelsPerInch, PrinterVPixelsPerInch: Integer;
    Image1Width, Image1Height, Image2Width, Image2Height: Integer;
  public
    { Public declarations }
    procedure PrintImage(Canvas: TCanvas; X, Y: Integer; gr: TGraphic; ToScreen: Boolean);
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

procedure TForm1.FormCreate(Sender: TObject);
var i: Integer;

  procedure GetPrinterPixelsPerInch;
  var DC: Cardinal;
  begin
    DC := RV_GetPrinterDC;
    PrinterHPixelsPerInch := GetDeviceCaps(DC, LOGPIXELSX);
    PrinterVPixelsPerInch := GetDeviceCaps(DC, LOGPIXELSY);
    DeleteDC(DC);
  end;

begin
  for i := 0 to 1000 do
    RichView1.AddNL('This is a printing test', Random(2), 0);
  RichView1.Format;

  Image1.Picture.LoadFromFile(ExtractFilePath(Application.ExeName)+'crocodile1.jpg');
  Image2.Picture.LoadFromFile(ExtractFilePath(Application.ExeName)+'crocodile2.jpg');


  GetPrinterPixelsPerInch;
  Image1Width  := Round(Image1.Picture.Graphic.Width  * PrinterHPixelsPerInch/Screen.PixelsPerInch);
  Image1Height := Round(Image1.Picture.Graphic.Height * PrinterVPixelsPerInch/Screen.PixelsPerInch);
  Image2Width  := Round(Image2.Picture.Graphic.Width  * PrinterHPixelsPerInch/Screen.PixelsPerInch);
  Image2Height := Round(Image2.Picture.Graphic.Height * PrinterVPixelsPerInch/Screen.PixelsPerInch);
  // reserving space for the first image
  RVPrint1.StartAt := Image1Height;
  RVPrint1.AssignSource(RichView1);
  RVPrint1.FormatPages(rvdoAll);
  RVPrint1.TransparentBackground := True;

  ScrollBar1.Min := 1;
  ScrollBar1.Max := RVPrint1.PagesCount;
  ScrollBar1.ShowHint := True;
end;

procedure TForm1.ScrollBar1Change(Sender: TObject);
begin
  RVPrintPreview1.PageNo := ScrollBar1.Position;
  ScrollBar1.Hint := Format('Page %d of %d', [ScrollBar1.Position, ScrollBar1.Max]);
end;

procedure TForm1.RVPrint1PagePrepaint(Sender: TRVPrint; PageNo: Integer;
  Canvas: TCanvas; Preview: Boolean; PageRect, PrintAreaRect: TRect);
begin
  if PageNo=1 then
    // Drawing image on top of the first page
    PrintImage(Canvas, (PrintAreaRect.Left+PrintAreaRect.Right-Image1Width) div 2,
      PrintAreaRect.Top, Image1.Picture.Graphic, Preview)
  else
    // Drawing image as a background for other pages
    PrintImage(Canvas,
      (PrintAreaRect.Left+PrintAreaRect.Right-Image2Width) div 2,
      (PrintAreaRect.Top+PrintAreaRect.Bottom-Image2Height) div 2,
      Image2.Picture.Graphic, Preview)
end;

procedure TForm1.PrintImage(Canvas: TCanvas; X, Y: Integer; gr: TGraphic;
   ToScreen: Boolean);
var bmp: TBitmap;
begin
  // RV_PictureToDevice understands only bitmaps
  bmp := TBitmap.Create;
  bmp.Assign(gr);
  RV_PictureToDevice(Canvas, X,Y, gr.Width, gr.Height,
    @(TCustomMainPtblRVData(RVPrint1.rv.RVData).PrnSad), bmp, ToScreen,
    RVStyle1.GraphicInterface);
  bmp.Free;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  RVPrint1.PrintPages(RVPrintPreview1.PageNo, RVPrintPreview1.PageNo,
    'Test Page #'+IntToStr(RVPrintPreview1.PageNo), 1, False);
end;

end.
